﻿# Migrating distributed vSwitch configurations from one vCenter to a new vCenter
# Written by: Gabrie van Zanten
# http://www.GabesVirtualWorld.com

function Get-dvSwitch{

	# This function was written by Luc Dekens
	# See: http://www.lucd.info/2009/10/12/dvswitch-scripting-part-2-dvportgroup/
		
	param([parameter(Position = 0, Mandatory = $true)][string]$DatacenterName,
	[parameter(Position = 1, Mandatory = $true)][string]$dvSwitchName)

	$dcNetFolder = Get-View (Get-Datacenter $DatacenterName | Get-View).NetworkFolder
	$found = $null
	foreach($net in $dcNetFolder.ChildEntity){
		if($net.Type -eq "VmwareDistributedVirtualSwitch"){
			$temp = Get-View $net
			if($temp.Name -eq $dvSwitchName){
				$found = $temp
			}
		}
	}
	$found
}

function Set-dvSwPgVLAN{

	# This function was written by Luc Dekens
	# See: http://www.lucd.info/2009/10/12/dvswitch-scripting-part-2-dvportgroup/
	
	param($dvSw, $dvPg, $vlanNr)

	$spec = New-Object VMware.Vim.DVPortgroupConfigSpec
	$spec.defaultPortConfig = New-Object VMware.Vim.VMwareDVSPortSetting
	$spec.DefaultPortConfig.vlan = New-Object VMware.Vim.VmwareDistributedVirtualSwitchVlanIdSpec
	$spec.defaultPortConfig.vlan.vlanId = $vlanNr

	$dvPg.UpdateViewData()
	$spec.ConfigVersion = $dvPg.Config.ConfigVersion

	$taskMoRef = $dvPg.ReconfigureDVPortgroup_Task($spec)

	$task = Get-View $taskMoRef
	while("running","queued" -contains $task.Info.State){
		$task.UpdateViewData("Info")
	}
}

function Get-dvSwPg{
	param($dvSw )

# Search for Portgroups
	$dvSw.Portgroup | %{Get-View -Id $_} 
	
}

# Report will be used to store the changes
$report=@()

# Ask what vCenter, datacenter and dvSwitch we're talking about
Disconnect-VIServer "*" -Force:$true
$vCenterOld = Read-Host "What is the name of the OLD vCenter: " 
$DatacenterName = Read-Host "What is the name of the datacenter on the OLD vCenter: " 
$OlddvSwitch = Read-Host "What is the name of the distributed vSwitch (dvSwitch) you want to move: "
$MovingHost = Read-Host "What is the name of the host you want to move: "

# Make the connection to vCenter
Connect-VIServer -Server $vCenterOld

# Read the dvSwitch and dvPortGroups
$dvSwitch = Get-dvSwitch $DatacenterName $OlddvSwitch
$dvPG = Get-dvSwPg $dvSwitch 

# Now create a (temporary) standard vSwitch with 128 ports. Remember, each VM needs one port and 128 might not be enough for you.
# The name of this temporary standard vSwitch will be 'vSwitch-Migrate'
New-VirtualSwitch -Name 'vSwitch-Migrate' -NumPorts 128 -VMHost $MovingHost 
$vSwitch = Get-VirtualSwitch -VMHost $MovingHost -Name 'vSwitch-Migrate'

foreach( $dvPGroup in $dvPG )
	{

	$VLANID = $dvPGroup.Config.DefaultPortConfig.Vlan.VlanId
	# Somehow the first line in $dvPGroup is some kind of header with 'VMware.Vim.NumericRange' in it. So I skip it 
	if( $VLANID -notmatch 'VMware.Vim.NumericRange')
		{
	
		# I want the new standard Portgroup to be named 'Mig-VLAN100' instead of 'VLAN100' 
		$NewPG = 'Mig-' + $dvPGroup.Name
	
		# Create a New standard portgroup
		Get-VirtualSwitch -VMHost $MovingHost -Name 'vSwitch-Migrate' | New-VirtualPortGroup -Name $NewPG -VLanId $VLANID 
		
		
		# Just to always know what was what, I keep track of old and new names
		# This is where you could add more settings from the olddvPG, like load balancing, number of ports, etc.
		$Conversion = "" | Select olddvSwitch, olddvPG, tmpvSwitch, tmpvPG, VLANID
		$Conversion.olddvswitch = $dvSwitch.Name
		$Conversion.olddvPG = $dvPGroup.Name
		$Conversion.tmpvSwitch = Get-VirtualSwitch -VMHost $MovingHost -Name 'vSwitch-Migrate' 
		$Conversion.tmpvPG = $NewPG
		$Conversion.VLANID = $VLANID
		$report += $Conversion
		
		}
	}

# Writing the info to CSV file
$report | Export-Csv "d:\csv\switch-list.csv" -NoTypeInformation

Write-Host "The dvSwitch and dvPortGroups have been exported to CSV file and a standard vSwitch with portgroups has been created."
Write-Host "Next Step is to run script 'Step02-Move-VM-to-vSwitch' which will move all VMs to standard vSwitches and portgroups"
